library shake_http_client;

import 'dart:convert';
import 'dart:typed_data';

import 'package:http/http.dart' as http;
import 'package:shake_http_client/shake_http_logger.dart';

/// Network requests tracker.
///
/// Makes all network requests visible on the dashboard.
class ShakeHttpClient implements http.Client {
  late http.Client client;
  late ShakeHttpLogger logger;

  ShakeHttpClient() : super() {
    client = http.Client();
    logger = ShakeHttpLogger();
  }

  @override
  Future<http.Response> delete(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    DateTime startTime = DateTime.now();
    return client
        .delete(url, headers: headers, body: body, encoding: encoding)
        .then((response) {
      logger.logNetworkRequest(startTime, response, body);
      return response;
    });
  }

  @override
  Future<http.Response> get(Uri url, {Map<String, String>? headers}) {
    DateTime startTime = DateTime.now();
    return client.get(url, headers: headers).then((response) {
      logger.logNetworkRequest(startTime, response, null);
      return response;
    });
  }

  @override
  Future<http.Response> head(Uri url, {Map<String, String>? headers}) {
    DateTime startTime = DateTime.now();
    return client.head(url, headers: headers).then((response) {
      logger.logNetworkRequest(startTime, response, null);
      return response;
    });
  }

  @override
  Future<http.Response> patch(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    DateTime startTime = DateTime.now();
    return client
        .patch(url, headers: headers, encoding: encoding)
        .then((response) {
      logger.logNetworkRequest(startTime, response, body);
      return response;
    });
  }

  @override
  Future<http.Response> post(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    DateTime startTime = DateTime.now();
    return client
        .post(url, headers: headers, encoding: encoding)
        .then((response) async {
      logger.logNetworkRequest(startTime, response, body);
      return response;
    });
  }

  @override
  Future<http.Response> put(Uri url,
      {Map<String, String>? headers, Object? body, Encoding? encoding}) {
    DateTime startTime = DateTime.now();
    return client
        .put(url, headers: headers, encoding: encoding)
        .then((response) {
      logger.logNetworkRequest(startTime, response, body);
      return response;
    });
  }

  @override
  Future<String> read(Uri url, {Map<String, String>? headers}) {
    return client.read(url, headers: headers).then((response) {
      return response;
    });
  }

  @override
  Future<Uint8List> readBytes(Uri url, {Map<String, String>? headers}) {
    return client.readBytes(url, headers: headers).then((response) {
      return response;
    });
  }

  @override
  Future<http.StreamedResponse> send(http.BaseRequest request) {
    return client.send(request);
  }

  @override
  void close() {
    client.close();
  }
}
